<?php
class XMLDatabase
{
	var $database; //database funcitons
	var $output;
	
	function XMLDatabase()
	{
		require_once ('Database_class.php');
		
		$this->database = new BiblioDatabase();
	}
	
	function readXMLDatabase($xmlFile)
	{
		$timestart = time();
		$doc = new DOMDocument();
		$doc->load($xmlFile);	
		
		$records = $doc->getElementsByTagName("record");
		
		//crear el archivo de salida
		foreach($records as $record) //para cada entrada
		{
			//$this->printRecord($record);
			//$this->saveRecord($record);
			//escribir los inserts sql
			
		}
		$timestop = time();
		$timeelapsed = $timestop-$timestart;
		$hours = floor($timeelapsed / 3600);
		$timeelapsed = $timeelapsed % 3600;
		$minutes = floor($timeelapsed / 60);
		$seconds = $timeelapsed % 60;
		
		echo "\nFinalizado en ".$hours." horas ".$minutes." minutos ".$seconds." segundos";
	}
	
	
	function getRefType($record)
	{
		//devuelve un int con el tipo de publicacion
		//17 journal
		//5 book chapter
		//6 whole book
		
		return $record->getElementsByTagName("ref-type")->item(0)->nodeValue;
	}
	
	function getAuthors($record)
	{
		//devuelve un array de strings con los nombres de los autores
		$authorNames = array();
					
		foreach($record->getElementsByTagName("author") as $author)
		{			
			$authorNames[] = trim($author->getElementsByTagName("style")->item(0)->nodeValue);
		}
		
		return $authorNames;
	}
	
	function joinAuthors($authors)
	{
		//recibe un array con autores y devuelve un string
		$joined = '';
		
		if (count($authors) == 1)
		{
			$joined = $authors[0];
		}
		else if (count($authors) == 2)
		{
			$joined = $authors[0].' and '.$authors[1];
		}
		else
		{
			for ($i=0; $i<count($authors)-2; $i++)
			{
				$joined .= $authors[$i].", ";
			}
			$joined .= $authors[count($authors) - 2].' and '.$authors[count($authors) - 1];			
		}
		
		return addslashes($joined);
	}
	
	function getTitle($record)
	{
		//devuelve un string con el titulo de la publicacion
		$tag = $record->getElementsByTagName("titles")->item(0)->getElementsByTagName("title");
		
		if ($tag->length == 0)
		{
			return -1;
		}
		
		$title = $tag->item(0)->getElementsByTagName("style")->item(0)->nodeValue;

		if (preg_match('/.+\D\d{1,3}$/', $title) != 0)
		{
			$title = $this->parseTitle($title);			
		}
		
		return addslashes(trim($title));
	}
	
	function parseTitle($title)
	{
		//devuelve el titulo sin los numeros finales de uno a tres numeros
		$titleLength = strlen($title);
		$count = 0;
		$limit = 3;
		$number = true;
		
		for ($i=$titleLength-1; $i>0 && $limit>0 && $number; $i--)
		{
			if (is_numeric($title[$i]))
			{
				$count++;
			}
			else
			{
				$number = false;
			}
			$limit --;
		}
		
		return substr($title, 0, ($titleLength - $count));
	}
	
	function getPublicationName($record)
	{
		//devuelve un string con el nombre de la publicacion
		$titles = $record->getElementsByTagName("secondary-title");
		
		if ($titles->length == 0) return "NULL";
		
		return addslashes(trim($titles->item(0)->getElementsByTagName("style")->item(0)->nodeValue));
	}
	
	function getPublisher($record)
	{
		//devuelve un string con el nombre de la editorial
		$publisher = $record->getElementsByTagName("publisher");
		
		if ($publisher->length == 0) return "NULL";
		
		$publisherName = trim($publisher->item(0)->getElementsByTagName("style")->item(0)->nodeValue);
		$publisherLocation = $this->getPublisherLocation($record);
		if ($publisherLocation != "NULL")
		{
			$publisherName .= " ".$publisherLocation;
		}
		
		return addslashes($publisherName);
	}
	
	function getPublisherLocation($record)
	{
		//devuelve un string con la localizacion de la editorial
		$publisher = $record->getElementsByTagName("pub-location");
		
		if ($publisher->length == 0) return "NULL";
		
		return addslashes(trim($publisher->item(0)->getElementsByTagName("style")->item(0)->nodeValue));
	}
	
	function getPages($record)
	{
		//devuelve un array de int con la pagina de inicio y la pagina de fin
		$pages = $record->getElementsByTagName("pages");
		$pageInfo = array();
		$pageInfo['start'] = "NULL";
		$pageInfo['end'] = "NULL";
		if ($pages->length == 0)
		{
			return $pageInfo;
		}
		
		$pageInfo['start'] = trim($pages->item(0)->getAttribute("start"));
		$pageInfo['end'] = trim($pages->item(0)->getAttribute("end"));
		if ($pageInfo['start'] == "")
		{
			$pageInfo['start'] = "NULL";
		}
		if ($pageInfo['end'] == "")
		{
			$pageInfo['end'] = "NULL";
		}
			
		return $pageInfo;
	}
	
	function getVolume($record)
	{
		//devuelve un int con el numero de volumen
		$volume = $record->getElementsByTagName("volume");
		
		if ($volume->length == 0) return "NULL";
		
		return trim($volume->item(0)->getElementsByTagName("style")->item(0)->nodeValue);
	}
	
	function getNumber($record)
	{
		//devuelve un int con el numero de publicacion
		$number = $record->getElementsByTagName("number");
		
		if ($number->length == 0) return "NULL";
		
		return trim($number->item(0)->getElementsByTagName("style")->item(0)->nodeValue);
	}
	
	function getDate($record)
	{
		//devuelve un array asociativo con la fecha de publicacion
		$year = $record->getElementsByTagName("year");
		
		$date = array();
		$date['year'] = trim($year->item(0)->getAttribute("Year"));
		$date['month'] = trim($year->item(0)->getAttribute("Month"));
		$date['day'] = trim($year->item(0)->getAttribute("Day"));
		
		return $date['year']."-".$date['month']."-".$date['day'];
	}
	
	function getUrl($record)
	{
		//devuelve un string con el enlace
		$url = $record->getElementsByTagName("url");
			
		if ($url->length == 0) return "NULL";
		
		if ($url->item(0)->getAttribute("ppv-ref") == "no")
			return trim($url->item(0)->nodeValue);
		else
			return "NULL";
	}
	
	function getPM($record)
	{
		$url = $record->getElementsByTagName("url");
		
		if ($url->length == 0) return "NULL";
		
		if ($url->item(0)->getAttribute("ppv-app") == "PM")
			return trim($url->item(0)->getAttribute("ppv-ut"));
		else
			return "NULL";
	}
	
	function printRecord($record)
	{
		//tipo de referencia
		echo "\nTIPO: ";
		echo $this->getRefType($record);
		
		//autores
		echo "\nAUTORES:";
		echo $this->joinAuthors($this->getAuthors($record));
		
		//titulo
		echo "\nTITULO: ";
		echo $this->getTitle($record);
				
		//journal 
		echo "\nREVISTA: ";
		echo $this->getPublicationName($record);
		
		//libro editorial		
		echo "\nEDITORIAL: ";
		echo $this->getPublisher($record)." ".$this->getPublisherLocation($record);
					
		//paginas
		echo "\nPAGINAS: ";
		$pages = $this->getPages($record);
		echo $pages['start']."-".$pages['end'];
					
		//volumen
		echo "\nVOLUMEN: ";
		echo $this->getVolume($record);
		
		//volumen
		echo "\nNUMERO: ";
		echo $this->getNumber($record);
		
		//fecha
		echo "\nFECHA: ";
		$date = $this->getDate($record);
		echo $date['year']."/".$date['month']."/".$date['day'];
		
		//PM
		echo "\nPM: ";
		$pm = $this->getPM($record);
		echo $pm;
		
		//url
		echo "\nURL: ";
		$url = $this->getUrl($record);
		echo $url;
		
		
		echo "\n---------------------------";
	}

	function saveRecord($record)
	{
		$isValid				= true;
		$refType				= $this->getRefType($record);
		$authors				= $this->joinAuthors($this->getAuthors($record));
		$title					= $this->getTitle($record);
		$date					= $this->getDate($record);
		$journal				= $this->getPublicationName($record);
		$publisher				= $this->getPublisher($record);
		$publisherLocation		= $this->getPublisherLocation($record);
		$volume					= $this->getVolume($record);
		$number					= $this->getNumber($record);
		$pages					= $this->getPages($record);
		$pm						= $this->getPM($record);
		$url					= $this->getUrl($record);
		
		if ($title == -1)
		{
			$isValid = false;
		}
		
		if ($isValid)
		{
			$this->database->saveReference($authors, $title, $date, $journal, $publisher, $volume, $pages['start'], $pages['end'], $pm, $url);
			//$this->printRecord($record);
		}
	}
	
	function createSQLFile()
	{
		$output = "INSERT INTO (`authors`, `title`, `journal`, `publisher`, `date`, `volume`, `pagestart`, `pageend`, `pmid`, `url`) VALUES ";
		
		return $output;
	}
	
	function createSQLInsert($record)
	{
		$output = "(".
		"'".$this->joinAuthors($this->getAuthors($record))."', ".
		"'".$this->getTitle($record)."', ".
		"'".$this->getDate($record)."', ".
		"'".$this->getPublicationName($record)."', ".
		"'".$this->getPublisher($record)."', ".
		"'".$this->getPublisherLocation($record)."', ".
		"'".$this->getVolume($record)."', ".
		"'". $this->getNumber($record)."', ".
		"'".$this->getPages($record)."', ".
		"'".$this->getPM($record)."', ".
		"'".$this->getUrl($record)."')";
		
		return $output;
	}
}
?>

<html>
<head></head>
<body>
<div style="width:800px;">
<pre>
<?php
	set_time_limit(0);
	$xmlDb = new XMLDatabase();
	//$xmlDb->readXMLDatabase('yo_3.xml');
	$xmlDb->readXMLDatabase('temp.xml');
?>
</pre>
</div>
</body>
</html>